@@ -14,7 +14,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/LINUX/osi_alloc.c,v 1.9.2.8 2003/05/28 23:49:21 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/LINUX/osi_alloc.c,v 1.12 2002/02/15 14:00:39 shadow Exp $");
 
 #include "../afs/sysincludes.h"
 #include "../afs/afsincludes.h"
@@ -24,7 +24,7 @@ RCSID("$Header: /cvs/openafs/src/afs/LINUX/osi_alloc.c,v 1.9.2.8 2003/05/28 23:4
 #include "../afs/afs_atomlist.h"
 #include "../afs/afs_lhash.h"
 
-#define MAX_KMALLOC_SIZE PAGE_SIZE /* Max we should alloc with kmalloc */
+#define MAX_KMALLOC_SIZE (131072-16) /* Max we can alloc in physmem */
 #define MAX_BUCKET_LEN 30 /* max. no. of entries per buckets we expect to see */
 #define STAT_INTERVAL 8192 /* we collect stats once every STAT_INTERVAL allocs*/
 
@@ -76,16 +76,17 @@ static int hash_equal(const void *a, const void *b)
  *  returns NULL if we failed to allocate memory.
  *  or pointer to memory if we succeeded.
  */
-static void *linux_alloc(unsigned int asize, int drop_glock)
+static void *linux_alloc(unsigned int asize)
 {
     void *new = NULL;
-    int max_retry = 10;
-    int haveGlock = ISAFS_GLOCK();
+    int has_afs_glock = ISAFS_GLOCK();
+
+    /* if global lock has been held save this info and unlock it. */
+    if (has_afs_glock)
+        AFS_GUNLOCK();
 
     /*  if we can use kmalloc use it to allocate the required memory. */
-    while(!new && max_retry)
-    {
-        if (asize <=  MAX_KMALLOC_SIZE) {
+    if (asize <  MAX_KMALLOC_SIZE) {
         new = (void *)(unsigned long)kmalloc(asize, 
 #ifdef GFP_NOFS
 					     GFP_NOFS
@@ -95,32 +96,30 @@ static void *linux_alloc(unsigned int asize, int drop_glock)
 					     );
         if (new) /* piggy back alloc type */
             (unsigned long)new |= KM_TYPE;
-        } else {
-            new = (void *)vmalloc(asize);
-	    if (new) /* piggy back alloc type */
-	        (unsigned long)new |= VM_TYPE;
+    }
+    if (!new) { /* otherwise use vmalloc  */
+	int max_wait = 10;
+        while (!(new = (void *)vmalloc(asize))) {
+            if (--max_wait <=0) {
+		break;
             }
-
-	if (!new) {
 #ifdef set_current_state
 	    set_current_state(TASK_INTERRUPTIBLE);
 #else
 	    current->state = TASK_INTERRUPTIBLE;
 #endif
-	    if (drop_glock && haveGlock) AFS_GUNLOCK();
 	    schedule_timeout(HZ);
-	    if (drop_glock && haveGlock) AFS_GLOCK();
-#ifdef set_current_state
-            set_current_state(TASK_RUNNING);
-#else
-            current->state = TASK_RUNNING;
-#endif
-            --max_retry;
         }
+	if (new) /* piggy back alloc type */
+	    (unsigned long)new |= VM_TYPE;
     }
     if (new)
 	memset(MEMADDR(new), 0, asize);
 
+    /* if the global lock had been held, lock it again. */
+    if (has_afs_glock)
+        AFS_GLOCK();
+
     return new;
 }
 
@@ -188,15 +187,9 @@ hash_verify(size_t index, unsigned key, void *data)
     int memtype;
 
     memtype = MEMTYPE(lmp->chunk);
-#if defined(AFS_SPARC64_LINUX24_ENV) || defined(AFS_I386_UMLINUX20_ENV)
-    if ((memtype == KM_TYPE) && (!VALID_PAGE(virt_to_page(lmp->chunk)))) {
-	printf("osi_linux_verify_alloced_memory: address 0x%x outside range, index=%d, key=%d\n", lmp->chunk, index, key);
-    }
-#else
     if ((memtype == KM_TYPE) && (AFS_LINUX_MAP_NR(lmp->chunk) > max_mapnr)) {
 	printf("osi_linux_verify_alloced_memory: address 0x%x outside range, index=%d, key=%d\n", lmp->chunk, index, key);
     }
-#endif
     
     if (memtype != KM_TYPE && memtype != VM_TYPE) {
 	printf("osi_linux_verify_alloced_memory: unknown type %d at 0x%x, index=%d\n",    memtype, lmp->chunk, index);
@@ -283,27 +276,27 @@ DECLARE_MUTEX(afs_linux_alloc_sem);
 struct semaphore afs_linux_alloc_sem = MUTEX;
 #endif
 
-void *osi_linux_alloc(unsigned int asize, int drop_glock)
+void *osi_linux_alloc(unsigned int asize)
 {
     void *new = NULL;
     struct osi_linux_mem *lmem;
 
-    new = linux_alloc(asize, drop_glock); /* get a chunk of memory of size asize */
-
-    if (!new) {
-	printf("afs_osi_Alloc: Can't vmalloc %d bytes.\n", asize);
-	return new;
-    }
-
     down(&afs_linux_alloc_sem);
 
-    /* allocator hasn't been initialized yet */
-    if (allocator_init == 0) {
+    if (allocator_init == 0) { /* allocator hasn't been initialized yet */
 	if (linux_alloc_init() == 0) {
 	    goto error;
 	 }
 	allocator_init = 1; /* initialization complete */
     }
+
+    up(&afs_linux_alloc_sem);
+    new = linux_alloc(asize); /* get a chunk of memory of size asize */
+    down(&afs_linux_alloc_sem);
+    if (!new) {
+	printf("afs_osi_Alloc: Can't vmalloc %d bytes.\n", asize);
+	goto error;
+    }
     
     /* get an atom to store the pointer to the chunk */
     lmem = (struct osi_linux_mem *)afs_atomlist_get(al_mem_pool);
