@@ -14,7 +14,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/UKERNEL/afs_usrops.c,v 1.9.2.1 2002/04/24 01:41:38 zacheiss Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/UKERNEL/afs_usrops.c,v 1.10 2002/02/01 20:30:11 kolya Exp $");
 
 
 #ifdef	UKERNEL
@@ -487,6 +487,13 @@ caddr_t x;
     }
 }
 
+int afs_osi_SleepSig(x)
+    caddr_t x;
+{
+    afs_osi_Sleep(x);
+    return 0;
+}
+
 int afs_osi_Wakeup(x)
 caddr_t x;
 {
@@ -4274,7 +4281,7 @@ int uafs_statmountpoint_r(char *path)
      return -1;
     }
 
-    avc = VTOAFS(vp);
+    avc = (struct vcache *) vp;
 
     r = avc->mvstat;
     VN_RELE(vp);
