@@ -22,7 +22,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_remove.c,v 1.4.4.10 2002/07/22 23:47:11 zacheiss Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_remove.c,v 1.10 2002/02/08 17:28:44 kolya Exp $");
 
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
 #include "../afs/afsincludes.h"	/* Afs-based standard headers */
@@ -50,9 +50,9 @@ static void FetchWholeEnchilada(avc, areq)
 #endif
 struct vrequest *areq;
 register struct vcache *avc; {
-    register afs_int32 nextChunk, pos;
+    register afs_int32 nextChunk;
     register struct dcache *tdc;
-    afs_int32 offset, len;
+    afs_size_t pos, offset, len;
 
     AFS_STATCNT(FetchWholeEnchilada);
     if ((avc->states & CStatd) == 0) return;	/* don't know size */
@@ -103,7 +103,6 @@ afsremove(adp, tdc, tvc, aname, acred, treqp)
     struct AFS_UCRED *acred; {
     register afs_int32 code;
     register struct conn *tc;
-    afs_int32 offset, len;
     struct AFSFetchStatus OutDirStatus;
     struct AFSVolSync tsync;
     XSTATS_DECLS
@@ -131,7 +130,10 @@ afsremove(adp, tdc, tvc, aname, acred, treqp)
     if (tvc) afs_symhint_inval(tvc);   /* XXX: don't really need to be so extreme */
 
     if (code) {
-	if (tdc) afs_PutDCache(tdc);
+	if (tdc) {
+	    ReleaseSharedLock(&tdc->lock);
+	    afs_PutDCache(tdc);
+	}
 	if (tvc) afs_PutVCache(tvc, WRITE_LOCK);
 
 	if (code < 0) {
@@ -145,6 +147,7 @@ afsremove(adp, tdc, tvc, aname, acred, treqp)
 	code = afs_CheckCode(code, treqp, 21);
 	return code;
     }
+    if (tdc) UpgradeSToWLock(&tdc->lock, 637);
     if (afs_LocalHero(adp, tdc, &OutDirStatus, 1)) {
 	/* we can do it locally */
 	code = afs_dir_Delete(&tdc->f.inode, aname);
@@ -154,6 +157,7 @@ afsremove(adp, tdc, tvc, aname, acred, treqp)
 	}
     }
     if (tdc) {
+	ReleaseWriteLock(&tdc->lock);
 	afs_PutDCache(tdc);	/* drop ref count */
     }
     ReleaseWriteLock(&adp->lock);
@@ -209,7 +213,7 @@ char *Tnam1;
 #ifdef	AFS_OSF_ENV
 afs_remove(ndp)
     struct nameidata *ndp; {
-    register struct vcache *adp = VTOAFS(ndp->ni_dvp);
+    register struct vcache *adp = (struct vcache *)ndp->ni_dvp;
     char *aname = ndp->ni_dent.d_name;
     struct ucred *acred = ndp->ni_cred;
 #else	/* AFS_OSF_ENV */
@@ -223,10 +227,9 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
     struct VenusFid unlinkFid;
     register afs_int32 code;
     register struct vcache *tvc;
-    afs_int32 offset, len;
+    afs_size_t offset, len;
     struct AFSFetchStatus OutDirStatus;
     struct AFSVolSync tsync;
-    struct afs_fakestat_state fakestate;
     XSTATS_DECLS
     OSI_VC_CONVERT(adp)
 
@@ -234,63 +237,26 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
     afs_Trace2(afs_iclSetp, CM_TRACE_REMOVE, ICL_TYPE_POINTER, adp,
 	       ICL_TYPE_STRING, aname);
 
-#ifdef	AFS_OSF_ENV
-    tvc = (struct vcache *)ndp->ni_vp;  /* should never be null */
-#endif
-
-    if (code = afs_InitReq(&treq, acred)) {
-#ifdef  AFS_OSF_ENV
-        afs_PutVCache(adp, 0);
-        afs_PutVCache(tvc, 0);
-#endif
-      return code;
-    }
-
-    afs_InitFakeStat(&fakestate);
-    code = afs_EvalFakeStat(&adp, &fakestate, &treq);
-    if (code) {
-	afs_PutFakeStat(&fakestate);
-#ifdef  AFS_OSF_ENV
-	afs_PutVCache(adp, 0);
-	afs_PutVCache(tvc, 0);
-#endif
-	return code;
-    }
-
     /* Check if this is dynroot */
-    if (afs_IsDynroot(adp)) {
-	code = afs_DynrootVOPRemove(adp, acred, aname);
-	afs_PutFakeStat(&fakestate);
-#ifdef  AFS_OSF_ENV
-	afs_PutVCache(adp, 0);
-	afs_PutVCache(tvc, 0);
-#endif
-	return code;
-    }
-    if (strlen(aname) > AFSNAMEMAX) {
-	afs_PutFakeStat(&fakestate);
-#ifdef  AFS_OSF_ENV
-	afs_PutVCache(adp, 0);
-	afs_PutVCache(tvc, 0);
-#endif
-	return ENAMETOOLONG;
-    }
+    if (afs_IsDynroot(adp))
+	return afs_DynrootVOPRemove(adp, acred, aname);
+
+    if (code = afs_InitReq(&treq, acred))
+      return code;
 tagain:
     code = afs_VerifyVCache(adp, &treq);
 #ifdef	AFS_OSF_ENV
-    tvc = VTOAFS(ndp->ni_vp);  /* should never be null */
+    tvc = (struct vcache *)ndp->ni_vp;  /* should never be null */
     if (code) {
 	afs_PutVCache(adp, 0);
 	afs_PutVCache(tvc, 0);
-	afs_PutFakeStat(&fakestate);
 	return afs_CheckCode(code, &treq, 22);
     }
 #else	/* AFS_OSF_ENV */
     tvc = (struct vcache *) 0;
     if (code) {
-	code = afs_CheckCode(code, &treq, 23);
-	afs_PutFakeStat(&fakestate);
-	return code;
+      code = afs_CheckCode(code, &treq, 23);
+      return code;
     }
 #endif
 
@@ -298,17 +264,13 @@ tagain:
       * fileserver
       */
     if ( adp->states & CRO ) {
-#ifdef  AFS_OSF_ENV
-        afs_PutVCache(adp, 0);
-        afs_PutVCache(tvc, 0);
-#endif
         code = EROFS;
-	afs_PutFakeStat(&fakestate);
 	return code;
     }
 
-    tdc	= afs_GetDCache(adp, 0,	&treq, &offset,	&len, 1);  /* test for error below */
+    tdc	= afs_GetDCache(adp, (afs_size_t) 0,	&treq, &offset,	&len, 1);  /* test for error below */
     ObtainWriteLock(&adp->lock,142);
+    ObtainSharedLock(&tdc->lock, 638);
 
     /*
      * Make sure that the data in the cache is current. We may have
@@ -317,8 +279,10 @@ tagain:
     if (!(adp->states & CStatd)
 	|| (tdc && !hsame(adp->m.DataVersion, tdc->f.versionNo))) {
 	ReleaseWriteLock(&adp->lock);
-	if (tdc)
+	if (tdc) {
+	    ReleaseSharedLock(&tdc->lock);
 	    afs_PutDCache(tdc);
+	}
 	goto tagain;
     }
 
@@ -374,7 +338,8 @@ tagain:
 	char *unlname = newname();
 
 	ReleaseWriteLock(&adp->lock);
-	code = afsrename(adp, aname, adp, unlname, acred, &treq);
+	if (tdc) ReleaseSharedLock(&tdc->lock);
+	code = afsrename(adp, aname, adp, unlname, acred);
 	Tnam1 = unlname;
 	if (!code) {
 	    tvc->mvid = (struct VenusFid *)unlname;
@@ -396,7 +361,6 @@ tagain:
 #ifdef	AFS_OSF_ENV
     afs_PutVCache(adp, WRITE_LOCK);
 #endif	/* AFS_OSF_ENV */
-    afs_PutFakeStat(&fakestate);
     return code;
 }
 
@@ -416,7 +380,7 @@ afs_remunlink(avc, doit)
     struct vrequest treq;
     struct VenusFid dirFid;
     register struct dcache *tdc;
-    afs_int32 offset, len, code=0;
+    afs_int32 code=0;
 
     if (NBObtainWriteLock(&avc->lock, 423))
 	return 0;
@@ -434,17 +398,6 @@ afs_remunlink(avc, doit)
 	    cred = avc->uncred;
 	    avc->uncred = NULL;
 
-#ifdef AFS_DARWIN_ENV
-           /* this is called by vrele (via VOP_INACTIVE) when the refcount
-              is 0. we can't just call VN_HOLD since vref will panic.
-              we can't just call osi_vnhold because a later AFS_RELE will call
-              vrele again, which will try to call VOP_INACTIVE again after
-              vn_locking the vnode. which would be fine except that our vrele
-              caller also locked the vnode... So instead, we just gimmick the
-              refcounts and hope nobody else can touch the file now */
-	    osi_Assert(VREFCOUNT(avc) == 0);
-	    VREFCOUNT_SET(avc, 1);
-#endif
 	    VN_HOLD(&avc->v);
 
 	    /* We'll only try this once. If it fails, just release the vnode.
@@ -464,8 +417,9 @@ afs_remunlink(avc, doit)
 	    if (adp) {
 		tdc = afs_FindDCache(adp, 0);
 		ObtainWriteLock(&adp->lock, 159);
+		if (tdc) ObtainSharedLock(&tdc->lock, 639);
 
-		/* afsremove releases the adp lock, and does vn_rele(avc) */
+		/* afsremove releases the adp & tdc locks, and does vn_rele(avc) */
 		code = afsremove(adp, tdc, avc, unlname, cred, &treq);
 		afs_PutVCache(adp, WRITE_LOCK);
 	    } else {
@@ -474,10 +428,6 @@ afs_remunlink(avc, doit)
 	    }
 	    osi_FreeSmallSpace(unlname);
 	    crfree(cred);
-#ifdef AFS_DARWIN_ENV
-	    osi_Assert(VREFCOUNT(avc) == 1);
-	    VREFCOUNT_SET(avc, 0);
-#endif
         }
     }
     else {
