@@ -17,7 +17,11 @@
  * afs_fsync
  */
 
-#include "../afs/param.h"	/* Should be always first */
+#include <afsconfig.h>
+#include "../afs/param.h"
+
+RCSID("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_write.c,v 1.16 2002/02/16 18:23:50 shadow Exp $");
+
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
 #include "../afs/afsincludes.h"	/* Afs-based standard headers */
 #include "../afs/afs_stats.h" /* statistics */
@@ -88,17 +92,20 @@ afs_MemWrite(avc, auio, aio, acred, noLock)
     register struct vcache *avc;
     struct uio *auio;
     int aio, noLock;
-    struct AFS_UCRED *acred; {
-    afs_int32 totalLength;
-    afs_int32 transferLength;
-    afs_int32 filePos;
+    struct AFS_UCRED *acred; 
+{
+    afs_size_t totalLength;
+    afs_size_t transferLength;
+    afs_size_t filePos;
+    afs_size_t offset, len;
+    afs_int32 tlen, trimlen;
     afs_int32 startDate;
     afs_int32 max;
     register struct dcache *tdc;
 #ifdef _HIGHC_
     volatile
 #endif
-    afs_int32 offset, len, error;
+    afs_int32 error;
     struct uio tuio;
     struct iovec *tvec;  /* again, should have define */
     char *tfile;
@@ -117,8 +124,9 @@ afs_MemWrite(avc, auio, aio, acred, noLock)
     error = 0;
     transferLength = 0;
     afs_Trace4(afs_iclSetp, CM_TRACE_WRITE, ICL_TYPE_POINTER, avc, 
-	       ICL_TYPE_INT32, filePos, ICL_TYPE_INT32, totalLength,
-	       ICL_TYPE_INT32, avc->m.Length);
+			ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(filePos),
+			ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(totalLength),
+			ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(avc->m.Length));
     if (!noLock) {
 	afs_MaybeWakeupTruncateDaemon();
 	ObtainWriteLock(&avc->lock,126);
@@ -167,7 +175,7 @@ afs_MemWrite(avc, auio, aio, acred, noLock)
 	return (EFBIG);
     }
 #endif
-#ifdef	AFS_VM_RDWR_ENV
+#ifdef AFS_VM_RDWR_ENV
     /*
      * If write is implemented via VM, afs_FakeOpen() is called from the
      * high-level write op.
@@ -189,20 +197,16 @@ afs_MemWrite(avc, auio, aio, acred, noLock)
 	 */
 	if (noLock) {
 	    tdc = afs_FindDCache(avc, filePos);
-	    if (tdc) {
-		offset = filePos - AFS_CHUNKTOBASE(tdc->f.chunk);
-		len = tdc->f.chunkBytes - offset;
-	    }
+	    if (tdc) ObtainWriteLock(&tdc->lock, 653);
 	} else if (afs_blocksUsed > (CM_WAITFORDRAINPCT*afs_cacheBlocks)/100) {
 	    tdc = afs_FindDCache(avc, filePos);
 	    if (tdc) {
+		ObtainWriteLock(&tdc->lock, 654);
 		if (!hsame(tdc->f.versionNo, avc->m.DataVersion) ||
-		    (tdc->flags & DFFetching)) {
+		    (tdc->dflags & DFFetching)) {
+		    ReleaseWriteLock(&tdc->lock);
 		    afs_PutDCache(tdc);
 		    tdc = NULL;
-		} else {
-		    offset = filePos - AFS_CHUNKTOBASE(tdc->f.chunk);
-		    len = tdc->f.chunkBytes - offset;
 		}
 	    }
 	    if (!tdc) {
@@ -221,9 +225,11 @@ afs_MemWrite(avc, auio, aio, acred, noLock)
 		}
 		avc->states |= CDirty;
 		tdc = afs_GetDCache(avc, filePos, &treq, &offset, &len, 4);
+		if (tdc) ObtainWriteLock(&tdc->lock, 655);
 	    }
 	} else {
 	    tdc = afs_GetDCache(avc, filePos, &treq, &offset, &len, 4);
+	    if (tdc) ObtainWriteLock(&tdc->lock, 656);
 	}
 	if (!tdc) {
 	    error = EIO;
@@ -236,9 +242,10 @@ afs_MemWrite(avc, auio, aio, acred, noLock)
 	if (!(tdc->f.states & DWriting)) {
 	    /* don't mark entry as mod if we don't have to */
 	    tdc->f.states |= DWriting;
-	    tdc->flags |= DFEntryMod;
+	    tdc->dflags |= DFEntryMod;
 	}
 	len = totalLength;	/* write this amount by default */
+	offset = filePos - AFS_CHUNKTOBASE(tdc->f.chunk);
 	max = AFS_CHUNKTOSIZE(tdc->f.chunk);	/* max size of this chunk */
 	if (max	<= len + offset)	{   /*if we'd go past the end of this chunk */
 	    /* it won't all fit in this chunk, so write as much
@@ -247,7 +254,8 @@ afs_MemWrite(avc, auio, aio, acred, noLock)
 	}
 	/* mung uio structure to be right for this transfer */
 	afsio_copy(auio, &tuio, tvec);
-	afsio_trim(&tuio, len);
+	trimlen = len;
+	afsio_trim(&tuio, trimlen);
 	tuio.afsio_offset = offset;
 
 	code = afs_MemWriteUIO(tdc->f.inode, &tuio);
@@ -257,15 +265,19 @@ afs_MemWrite(avc, auio, aio, acred, noLock)
 	    afs_MemCacheTruncate(tdc->f.inode, 0);
 	    afs_stats_cmperf.cacheCurrDirtyChunks--;
 	    afs_indexFlags[tdc->index] &= ~IFDataMod;    /* so it does disappear */
+	    ReleaseWriteLock(&tdc->lock);
 	    afs_PutDCache(tdc);
 	    break;
 	}
 	/* otherwise we've written some, fixup length, etc and continue with next seg */
 	len = len - tuio.afsio_resid; /* compute amount really transferred */
-	afsio_skip(auio, len);	    /* advance auio over data written */
+	tlen = len;
+	afsio_skip(auio, tlen);	    /* advance auio over data written */
 	/* compute new file size */
-	if (offset + len > tdc->f.chunkBytes)
-	    afs_AdjustSize(tdc, offset+len);
+	if (offset + len > tdc->f.chunkBytes) {
+	    afs_int32 tlength = offset+len;
+	    afs_AdjustSize(tdc, tlength);
+	}
 	totalLength -= len;
 	transferLength += len;
 	filePos += len;
@@ -273,10 +285,16 @@ afs_MemWrite(avc, auio, aio, acred, noLock)
         /* afs_xwrite handles setting m.Length */
         osi_Assert(filePos <= avc->m.Length);
 #else
-	if (filePos > avc->m.Length)
+	if (filePos > avc->m.Length) {
+	    afs_Trace4(afs_iclSetp, CM_TRACE_SETLENGTH,
+		ICL_TYPE_STRING, __FILE__,
+		ICL_TYPE_LONG, __LINE__,
+		ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(avc->m.Length),
+		ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(filePos));
 	    avc->m.Length = filePos;
+	}
 #endif
-#ifndef	AFS_VM_RDWR_ENV
+#ifndef AFS_VM_RDWR_ENV
 	/*
 	 * If write is implemented via VM, afs_DoPartialWrite() is called from
 	 * the high-level write op.
@@ -285,11 +303,13 @@ afs_MemWrite(avc, auio, aio, acred, noLock)
 	    code = afs_DoPartialWrite(avc, &treq);
 	    if (code) {
 		error = code;
+		ReleaseWriteLock(&tdc->lock);
 		afs_PutDCache(tdc);
 		break;
 	    }
 	}
 #endif
+	ReleaseWriteLock(&tdc->lock);
 	afs_PutDCache(tdc);
     }
 #ifndef	AFS_VM_RDWR_ENV
@@ -315,17 +335,21 @@ afs_UFSWrite(avc, auio, aio, acred, noLock)
     register struct vcache *avc;
     struct uio *auio;
     int aio, noLock;
-    struct AFS_UCRED *acred; {
-    afs_int32 totalLength;
-    afs_int32 transferLength;
-    afs_int32 filePos;
+    struct AFS_UCRED *acred; 
+{
+    afs_size_t totalLength;
+    afs_size_t transferLength;
+    afs_size_t filePos;
+    afs_size_t offset, len;
+    afs_int32  tlen;
+    afs_int32  trimlen;
     afs_int32 startDate;
     afs_int32 max;
     register struct dcache *tdc;
 #ifdef _HIGHC_
     volatile
 #endif
-    afs_int32 offset, len, error;
+    afs_int32 error;
     struct uio tuio;
     struct iovec *tvec;  /* again, should have define */
     struct osi_file *tfile;
@@ -345,8 +369,9 @@ afs_UFSWrite(avc, auio, aio, acred, noLock)
     error = 0;
     transferLength = 0;
     afs_Trace4(afs_iclSetp, CM_TRACE_WRITE, ICL_TYPE_POINTER, avc, 
-	       ICL_TYPE_INT32, filePos, ICL_TYPE_INT32, totalLength,
-	       ICL_TYPE_INT32, avc->m.Length);
+			ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(filePos),
+			ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(totalLength),
+			ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(avc->m.Length));
     if (!noLock) {
 	afs_MaybeWakeupTruncateDaemon();
 	ObtainWriteLock(&avc->lock,556);
@@ -412,20 +437,16 @@ afs_UFSWrite(avc, auio, aio, acred, noLock)
 	/* read the cached info */
 	if (noLock) {
 	    tdc = afs_FindDCache(avc, filePos);
-	    if (tdc) {
-		offset = filePos - AFS_CHUNKTOBASE(tdc->f.chunk);
-		len = tdc->f.chunkBytes - offset;
-	    }
+	    if (tdc) ObtainWriteLock(&tdc->lock, 657);
 	} else if (afs_blocksUsed > (CM_WAITFORDRAINPCT*afs_cacheBlocks)/100) {
 	    tdc = afs_FindDCache(avc, filePos);
 	    if (tdc) {
+		ObtainWriteLock(&tdc->lock, 658);
 		if (!hsame(tdc->f.versionNo, avc->m.DataVersion) ||
-		    (tdc->flags & DFFetching)) {
+		    (tdc->dflags & DFFetching)) {
+		    ReleaseWriteLock(&tdc->lock);
 		    afs_PutDCache(tdc);
 		    tdc = NULL;
-		} else {
-		    offset = filePos - AFS_CHUNKTOBASE(tdc->f.chunk);
-		    len = tdc->f.chunkBytes - offset;
 		}
 	    }
 	    if (!tdc) {
@@ -444,9 +465,11 @@ afs_UFSWrite(avc, auio, aio, acred, noLock)
 		}
 		avc->states |= CDirty;
 		tdc = afs_GetDCache(avc, filePos, &treq, &offset, &len, 4);
+		if (tdc) ObtainWriteLock(&tdc->lock, 659);
 	    }
 	} else {
 	    tdc = afs_GetDCache(avc, filePos, &treq, &offset, &len, 4);
+	    if (tdc) ObtainWriteLock(&tdc->lock, 660);
 	}
 	if (!tdc) {
 	    error = EIO;
@@ -459,10 +482,11 @@ afs_UFSWrite(avc, auio, aio, acred, noLock)
 	if (!(tdc->f.states & DWriting)) {
 	    /* don't mark entry as mod if we don't have to */
 	    tdc->f.states |= DWriting;
-	    tdc->flags |= DFEntryMod;
+	    tdc->dflags |= DFEntryMod;
 	}
 	tfile = (struct osi_file *)osi_UFSOpen(tdc->f.inode);
 	len = totalLength;	/* write this amount by default */
+	offset = filePos - AFS_CHUNKTOBASE(tdc->f.chunk);
 	max = AFS_CHUNKTOSIZE(tdc->f.chunk);	/* max size of this chunk */
 	if (max	<= len + offset)	{   /*if we'd go past the end of this chunk */
 	    /* it won't all fit in this chunk, so write as much
@@ -471,7 +495,8 @@ afs_UFSWrite(avc, auio, aio, acred, noLock)
 	}
 	/* mung uio structure to be right for this transfer */
 	afsio_copy(auio, &tuio, tvec);
-	afsio_trim(&tuio, len);
+	trimlen = len;
+	afsio_trim(&tuio, trimlen);
 	tuio.afsio_offset = offset;
 #ifdef	AFS_AIX_ENV
 #ifdef	AFS_AIX41_ENV
@@ -528,8 +553,24 @@ afs_UFSWrite(avc, auio, aio, acred, noLock)
 	AFS_GUNLOCK();
 	code = osi_file_uio_rdwr(tfile, &tuio, UIO_WRITE);
 	AFS_GLOCK();
+#else
+#if defined(AFS_DARWIN_ENV)
+        AFS_GUNLOCK();
+        VOP_LOCK(tfile->vnode, LK_EXCLUSIVE, current_proc());
+        code = VOP_WRITE(tfile->vnode, &tuio, 0, &afs_osi_cred);
+        VOP_UNLOCK(tfile->vnode, 0, current_proc());
+        AFS_GLOCK();
+#else
+#if defined(AFS_FBSD_ENV)
+        AFS_GUNLOCK();
+        VOP_LOCK(tfile->vnode, LK_EXCLUSIVE, curproc);
+        code = VOP_WRITE(tfile->vnode, &tuio, 0, &afs_osi_cred);
+        VOP_UNLOCK(tfile->vnode, 0, curproc);
+        AFS_GLOCK();
 #else
 	code = VOP_RDWR(tfile->vnode, &tuio, UIO_WRITE, 0, &afs_osi_cred);
+#endif /* AFS_FBSD_ENV */
+#endif /* AFS_DARWIN_ENV */
 #endif /* AFS_LINUX20_ENV */
 #endif /* AFS_HPUX100_ENV */
 #endif /* AFS_OSF_ENV */
@@ -543,16 +584,20 @@ afs_UFSWrite(avc, auio, aio, acred, noLock)
 	    afs_AdjustSize(tdc,	0);	/* sets f.chunkSize to 0 */
 	    afs_stats_cmperf.cacheCurrDirtyChunks--;
 	    afs_indexFlags[tdc->index] &= ~IFDataMod;    /* so it does disappear */
-	    afs_PutDCache(tdc);
 	    afs_CFileClose(tfile);
+	    ReleaseWriteLock(&tdc->lock);
+	    afs_PutDCache(tdc);
 	    break;
 	}
 	/* otherwise we've written some, fixup length, etc and continue with next seg */
 	len = len - tuio.afsio_resid; /* compute amount really transferred */
-	afsio_skip(auio, len);	    /* advance auio over data written */
+	tlen = len;
+	afsio_skip(auio, tlen);	    /* advance auio over data written */
 	/* compute new file size */
-	if (offset + len > tdc->f.chunkBytes)
-	    afs_AdjustSize(tdc, offset+len);
+	if (offset + len > tdc->f.chunkBytes) {
+	    afs_int32 tlength = offset+len;
+	    afs_AdjustSize(tdc, tlength);
+	}
 	totalLength -= len;
 	transferLength += len;
 	filePos += len;
@@ -561,6 +606,11 @@ afs_UFSWrite(avc, auio, aio, acred, noLock)
         osi_Assert(filePos <= avc->m.Length);
 #else
 	if (filePos > avc->m.Length) {
+	    afs_Trace4(afs_iclSetp, CM_TRACE_SETLENGTH,
+		ICL_TYPE_STRING, __FILE__,
+		ICL_TYPE_LONG, __LINE__,
+		ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(avc->m.Length),
+		ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(filePos));
 	    avc->m.Length = filePos;
 	}
 #endif
@@ -574,11 +624,13 @@ afs_UFSWrite(avc, auio, aio, acred, noLock)
 	    code = afs_DoPartialWrite(avc, &treq);
 	    if (code) {
 		error = code;
+		ReleaseWriteLock(&tdc->lock);
 		afs_PutDCache(tdc);
 		break;
 	    }
 	}
 #endif
+	ReleaseWriteLock(&tdc->lock);
 	afs_PutDCache(tdc);
     }
 #ifndef	AFS_VM_RDWR_ENV
@@ -601,6 +653,9 @@ afs_UFSWrite(avc, auio, aio, acred, noLock)
      * If write is implemented via VM, afs_fsync() is called from the high-level
      * write op.
      */
+#if defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
+     if (noLock && (aio & IO_SYNC)) {
+#else 
 #ifdef	AFS_HPUX_ENV
     /* On hpux on synchronous writes syncio will be set to IO_SYNC. If
      * we're doing them because the file was opened with O_SYNCIO specified,
@@ -609,6 +664,7 @@ afs_UFSWrite(avc, auio, aio, acred, noLock)
     if (noLock && ((aio & IO_SYNC) | (auio->uio_fpflags & FSYNCIO))) {    
 #else
     if (noLock && (aio & FSYNC)) {
+#endif
 #endif
 	if (!AFS_NFSXLATORREQ(acred))
 	    afs_fsync(avc, acred);
@@ -627,7 +683,7 @@ struct vrequest *areq; {
 	return 0;	/* nothing to do */
     /* otherwise, call afs_StoreDCache (later try to do this async, if possible) */
     afs_Trace2(afs_iclSetp, CM_TRACE_PARTIALWRITE, ICL_TYPE_POINTER, avc,
-	       ICL_TYPE_INT32, avc->m.Length);
+		ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(avc->m.Length));
 #if	defined(AFS_SUN5_ENV)
     code = afs_StoreAllSegments(avc, areq, AFS_ASYNC | AFS_VMSYNC_INVAL);
 #else
@@ -638,7 +694,7 @@ struct vrequest *areq; {
 
 
 
-#if !defined (AFS_AIX_ENV) && !defined (AFS_HPUX_ENV) && !defined (AFS_SUN5_ENV) && !defined(AFS_SGI_ENV) && !defined(AFS_LINUX20_ENV)
+#if !defined (AFS_AIX_ENV) && !defined (AFS_HPUX_ENV) && !defined (AFS_SUN5_ENV) && !defined(AFS_SGI_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_DARWIN_ENV) && !defined(AFS_FBSD_ENV)
 #ifdef AFS_DUX50_ENV
 #define vno_close(X) vn_close((X), 0, NOCRED)
 #elif defined(AFS_DUX40_ENV)
@@ -821,8 +877,9 @@ afs_close(OSI_VC_ARG(avc), aflags, acred)
 #endif
 	    /* at least one daemon is idle, so ask it to do the store.
 		Also, note that  we don't lock it any more... */
-	    tb = afs_BQueue(BOP_STORE, avc, 0, 1, acred, (long)acred->cr_uid,
-			    0L, 0L, 0L);
+            tb = afs_BQueue(BOP_STORE, avc, 0, 1, acred,
+                                (afs_size_t) acred->cr_uid, (afs_size_t) 0,
+                                (void *) 0);
 	    /* sleep waiting for the store to start, then retrieve error code */
 	    while ((tb->flags & BUVALID) == 0) {
 		tb->flags |= BUWAIT;
@@ -893,7 +950,7 @@ afs_close(OSI_VC_ARG(avc), aflags, acred)
 	ReleaseWriteLock(&avc->lock);
     }
 #ifdef	AFS_OSF_ENV
-    if ((avc->vrefCount <= 2) && (avc->states & CUnlinked)) {
+    if ((VREFCOUNT(avc) <= 2) && (avc->states & CUnlinked)) {
 	afs_remunlink(avc, 1);	/* ignore any return code */
     }
 #endif
