@@ -116,13 +116,13 @@ struct sysname_info {
 struct brequest {
     struct vcache *vnode;	    /* vnode to use, with vrefcount bumped */
     struct AFS_UCRED *cred;	    /* credentials to use for operation */
-    long parm[BPARMS];		    /* random parameters - long's work for
-				     * both 32 and 64 bit platforms.
-				     */
-    afs_int32 code;			    /* return code */
+    afs_size_t size_parm[BPARMS];   /* random parameters */
+    void *ptr_parm[BPARMS];	    /* pointer parameters */
+    afs_int32 code;		    /* return code */
     short refCount;		    /* use counter for this structure */
     char opcode;		    /* what to do (store, fetch, etc) */
     char flags;			    /* free, etc */
+    afs_int32 ts;		    /* counter "timestamp" */
 };
 
 struct SecretToken {
@@ -202,7 +202,7 @@ struct cell {
     short states;			    /* state flags */
     short cellIndex;			    /* relative index number per cell */
     time_t timeout;			    /* data expire time, if non-zero */
-    struct cell *alias;			    /* what this cell is an alias for */
+    char *realName;			    /* who this cell is an alias for */
 };
 
 #define	afs_PutCell(cellp, locktype)
@@ -329,6 +329,11 @@ struct srvAddr {
 #define	SRVR_ISDOWN			0x20
 #define	SRVR_MULTIHOMED			0x40
 #define	SRVR_ISGONE			0x80
+#define	SNO_INLINEBULK			0x100
+#define SNO_64BIT                       0x200 
+
+#define afs_serverSetNo64Bit(s) ((s)->srvr->server->flags |= SNO_64BIT)
+#define afs_serverHasNo64Bit(s) ((s)->srvr->server->flags & SNO_64BIT)
 
 struct server {
     union {
@@ -356,7 +361,7 @@ struct server {
     afs_int32 numDowntimeIncidents;	/* # (completed) downtime incidents */
     afs_int32 sumOfDowntimes;	/* Total downtime experienced, in seconds */
     struct srvAddr *addr;
-    char flags;			/* Misc flags*/
+    afs_uint32 flags;			/* Misc flags*/
 };
 
 #define	afs_PutServer(servp, locktype)	
@@ -500,7 +505,6 @@ struct SimpleLocks {
 #endif
 #define CUnique		0x00001000	/* vc's uniquifier - latest unifiquier for fid */
 #define CForeign	0x00002000	/* this is a non-afs vcache */
-#define CHasPages	0x00004000
 #define CUnlinked	0x00010000
 #define CBulkStat	0x00020000	/* loaded by a bulk stat, and not ref'd since */
 #define CUnlinkedDel	0x00040000
@@ -529,15 +533,27 @@ struct SimpleLocks {
 #define VREFCOUNT_SET(v, c)	atomic_set(&((vnode_t *) v)->v_count, c)
 #define VREFCOUNT_DEC(v)	atomic_dec(&((vnode_t *) v)->v_count)
 #define VREFCOUNT_INC(v)	atomic_inc(&((vnode_t *) v)->v_count)
+#define DLOCK()      spin_lock(&dcache_lock)
+#define DUNLOCK()    spin_unlock(&dcache_lock)
+#define DGET(d)      dget_locked(d)
+#define DCOUNT(d)    atomic_read(&(d)->d_count)
 #else
 #define VREFCOUNT(v)		((v)->vrefCount)
 #define VREFCOUNT_SET(v, c)	(v)->vrefCount = c;
 #define VREFCOUNT_DEC(v)	(v)->vrefCount--;
 #define VREFCOUNT_INC(v)	(v)->vrefCount++;
+#define DLOCK()
+#define DUNLOCK()
+#define DGET(d)      dget(d)
+#define DCOUNT(d)    ((d)->d_count)
 #endif
 
 #define	AFS_MAXDV   0x7fffffff	    /* largest dataversion number */
+#ifdef AFS_64BIT_CLIENT
+#define AFS_NOTRUNC 0x7fffffffffffffffLL  /* largest positive int64 number */
+#else /* AFS_64BIT_CLIENT */
 #define	AFS_NOTRUNC 0x7fffffff	    /* largest dataversion number */
+#endif /* AFS_64BIT_CLIENT */
 
 extern afs_int32 vmPageHog; /* counter for # of vnodes which are page hogs. */
 
@@ -549,8 +565,8 @@ struct	vtodc
 	struct dcache * dc;
 	afs_uint32		stamp;
 	struct osi_file * f;
-	afs_uint32		minLoc;	/* smallest offset into dc. */
-	afs_uint32		len;	/* largest offset into dc. */
+	afs_offs_t		minLoc;	/* smallest offset into dc. */
+	afs_offs_t		len;	/* largest offset into dc. */
 	};
 
 extern afs_uint32 afs_stampValue;		/* stamp for pair's usage */
@@ -567,7 +583,7 @@ struct vcache {
     struct vcache *hnext;		/* Hash next */
     struct VenusFid fid;
     struct mstat {
-        afs_uint32 Length;
+        afs_size_t Length;
         afs_hyper_t DataVersion;
         afs_uint32 Date;
         afs_uint32 Owner;
@@ -582,11 +598,11 @@ struct vcache {
      * Do not try to get the vcache lock when the vlock is held */
     afs_rwlock_t vlock;
 #endif /* defined(AFS_SUN5_ENV) */
-#if defined(AFS_SUN_ENV) || defined(AFS_ALPHA_ENV) || defined(AFS_DARWIN_ENV)
 #if	defined(AFS_SUN5_ENV)
     krwlock_t rwlock;
     struct cred *credp;
 #endif
+#if defined(AFS_SUN_ENV) || defined(AFS_ALPHA_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
     afs_bozoLock_t pvnLock;	/* see locks.x */
 #endif
 #ifdef	AFS_AIX32_ENV
@@ -600,20 +616,23 @@ struct vcache {
 #endif
 #ifdef AFS_DARWIN_ENV
     struct lock__bsd__      rwlock;
+#endif
+#ifdef AFS_FBSD_ENV
+    struct lock      rwlock;
 #endif
     afs_int32 parentVnode;			/* Parent dir, if a file. */
     afs_int32 parentUnique;
     struct VenusFid *mvid;		/* Either parent dir (if root) or root (if mt pt) */
     char *linkData;			/* Link data if a symlink. */
-    afs_hyper_t flushDV;			/* data version last flushed from text */
+    afs_hyper_t flushDV;		/* data version last flushed from text */
     afs_hyper_t mapDV;			/* data version last flushed from map */
-    afs_uint32 truncPos;			/* truncate file to this position at next store */
+    afs_size_t truncPos;		/* truncate file to this position at next store */
     struct server *callback;		/* The callback host, if any */
-    afs_uint32 cbExpires;			/* time the callback expires */
+    afs_uint32 cbExpires;		/* time the callback expires */
     struct afs_q callsort;              /* queue in expiry order, sort of */
     struct axscache *Access;            /* a list of cached access bits */
-    afs_int32 anyAccess;			/* System:AnyUser's access to this. */
-    afs_int32 last_looker;                  /* pag/uid from last lookup here */
+    afs_int32 anyAccess;		/* System:AnyUser's access to this. */
+    afs_int32 last_looker;              /* pag/uid from last lookup here */
 #if	defined(AFS_SUN5_ENV)
     afs_int32 activeV;
 #endif /* defined(AFS_SUN5_ENV) */
@@ -769,11 +788,12 @@ struct cm_initparams {
 /* struct dcache states bits */
 #define	DWriting    8		/* file being written (used for cache validation) */
 
-/* dcache flags */
-#define	DFNextStarted	1	/* next chunk has been prefetched already */
-#define	DFEntryMod	2	/* has entry itself been modified? */
-#define	DFFetching	4	/* file is currently being fetched */
-#define	DFWaiting	8	/* someone waiting for file */
+/* dcache data flags */
+#define	DFEntryMod	0x02	/* has entry itself been modified? */
+#define	DFFetching	0x04	/* file is currently being fetched */
+
+/* dcache meta flags */
+#define	DFNextStarted	0x01	/* next chunk has been prefetched already */
 #define	DFFetchReq	0x10	/* someone is waiting for DFFetching to go on */
 
 
@@ -829,21 +849,21 @@ struct afs_fheader {
 struct fcache {
     struct VenusFid fid;	/* Fid for this file */
     afs_int32 modTime;		/* last time this entry was modified */
-    afs_hyper_t versionNo;		/* Associated data version number */
-    afs_int32 chunk;			/* Relative chunk number */
+    afs_hyper_t versionNo;	/* Associated data version number */
+    afs_int32 chunk;		/* Relative chunk number */
 #if defined(AFS_SGI61_ENV) || defined(AFS_SUN57_64BIT_ENV)
     /* Using ino64_t here so that user level debugging programs compile
      * the size correctly.
      */
-    ino64_t inode;			/* Unix inode for this chunk */
+    ino64_t inode;		/* Unix inode for this chunk */
 #else
 #if defined(AFS_LINUX_64BIT_KERNEL)
-    long inode;	 			/* Unix inode for this chunk */
+    long inode;	 		/* Unix inode for this chunk */
 #else
-    afs_int32 inode;			/* Unix inode for this chunk */
+    afs_int32 inode;		/* Unix inode for this chunk */
 #endif
 #endif
-    afs_int32 chunkBytes;		/* Num bytes in this chunk */
+    afs_int32 chunkBytes;	/* Num bytes in this chunk */
     char states;		/* Has this chunk been modified? */
 };
 
@@ -857,13 +877,47 @@ struct fcache {
 /* kept in memory */
 struct dcache {
     struct afs_q lruq;		/* Free queue for in-memory images */
-    afs_rwlock_t lock;		/* XXX */
+    afs_rwlock_t lock;		/* Protects validPos, some f */
+    afs_rwlock_t tlock;		/* Atomizes updates to refCount */
+    afs_rwlock_t mflock;	/* Atomizes accesses/updates to mflags */
+    afs_size_t validPos;	/* number of valid bytes during fetch */
+    afs_int32 index;		/* The index in the CacheInfo file*/
     short refCount;		/* Associated reference count. */
-    afs_int32 index;			/* The index in the CacheInfo file*/
-    short flags;		/* more flags bits */
-    afs_int32 validPos;		/* number of valid bytes during fetch */
+    char dflags;		/* Data flags */
+    char mflags;		/* Meta flags */
     struct fcache f;		/* disk image */
-    afs_int32 stamp; 		/* used with vtodc struct for hints */
+    afs_int32 stamp;		/* used with vtodc struct for hints */
+
+    /*
+     * Locking rules:
+     *
+     * dcache.lock protects the actual contents of the cache file (in
+     * f.inode), subfields of f except those noted below, dflags and
+     * validPos.
+     *
+     * dcache.tlock is used to make atomic updates to refCount.  Zero
+     * refCount dcache entries are protected by afs_xdcache instead of
+     * tlock.
+     *
+     * dcache.mflock is used to access and update mflags.  It cannot be
+     * held without holding the corresponding dcache.lock.  Updating
+     * mflags requires holding dcache.lock(R) and dcache.mflock(W), and
+     * checking for mflags requires dcache.lock(R) and dcache.mflock(R).
+     * Note that dcache.lock(W) gives you the right to update mflags,
+     * as dcache.mflock(W) can only be held with dcache.lock(R).
+     *
+     * dcache.stamp is protected by the associated vcache lock, because
+     * it's only purpose is to establish correspondence between vcache
+     * and dcache entries.
+     *
+     * dcache.index, dcache.f.fid, dcache.f.chunk and dcache.f.inode are
+     * write-protected by afs_xdcache and read-protected by refCount.
+     * Once an entry is referenced, these values cannot change, and if
+     * it's on the free list (with refCount=0), it can be reused for a
+     * different file/chunk.  These values can only be written while
+     * holding afs_xdcache(W) and allocating this dcache entry (thereby
+     * ensuring noone else has a refCount on it).
+     */
 };
 /* this is obsolete and should be removed */
 #define ihint stamp 
@@ -873,15 +927,9 @@ struct dcache {
     do { \
 	(x)->f.fid.Fid.Unique = 0; \
 	afs_indexUnique[(x)->index] = 0; \
-	(x)->flags |= DFEntryMod; \
+	(x)->dflags |= DFEntryMod; \
     } while(0)
 
-/*
- * Convenient release macro for use when afs_PutDCache would cause
- * deadlock on afs_xdcache lock
- */
-#define lockedPutDCache(ad) ((ad)->refCount--)
-
 /* FakeOpen and Fake Close used to be real subroutines.  They're only used in
  * sun_subr and afs_vnodeops, and they're very frequently called, so I made 
  * them into macros.  They do:
@@ -959,7 +1007,9 @@ extern struct brequest afs_brs[NBRS];		/* request structures */
 #define	FVHash(acell,avol)  (((avol)+(acell)) & (NFENTRIES-1))
 
 extern struct cell	    *afs_GetCell();
+extern struct cell	    *afs_GetCellNoLock();
 extern struct cell	    *afs_GetCellByName();
+extern struct cell	    *afs_GetCellByName2();
 extern struct cell	    *afs_GetCellByIndex();
 extern struct unixuser	    *afs_GetUser();
 extern struct volume	    *afs_GetVolume();
@@ -967,7 +1017,7 @@ extern struct volume	    *afs_GetVolumeByName();
 extern struct conn	    *afs_Conn();
 extern struct conn	    *afs_ConnByHost();
 extern struct conn	    *afs_ConnByMHosts();
-extern afs_int32		    afs_NewCell();
+extern afs_int32	    afs_NewCell();
 extern struct dcache	    *afs_GetDCache();
 extern struct dcache	    *afs_FindDCache();
 extern struct dcache	    *afs_NewDCache();
@@ -1003,6 +1053,8 @@ extern void afs_PutDynroot();
 extern int afs_DynrootNewVnode();
 extern int afs_SetDynrootEnable();
 extern int afs_GetDynrootEnable();
+extern int afs_DynrootVOPSymlink();
+extern int afs_DynrootVOPRemove();
 
 
 /* Performance hack - we could replace VerifyVCache2 with the appropriate
