@@ -16,7 +16,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/afs_callback.c,v 1.6.2.5 2002/04/21 20:23:56 zacheiss Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/afs_callback.c,v 1.9 2001/11/01 04:01:22 shadow Exp $");
 
 #include "../afs/sysincludes.h" /*Standard vendor system headers*/
 #include "../afs/afsincludes.h" /*AFS-based standard headers*/
@@ -153,7 +153,11 @@ int SRXAFSCB_GetCE(a_call, a_index, a_result)
     a_result->lock.pid_writer = 0;
     a_result->lock.src_indicator = 0;
 #endif /* AFS_OSF20_ENV */
+#ifdef AFS_64BIT_CLIENT
+    a_result->Length = (afs_int32) tvc->m.Length & 0xffffffff;
+#else /* AFS_64BIT_CLIENT */
     a_result->Length = tvc->m.Length;
+#endif /* AFS_64BIT_CLIENT */
     a_result->DataVersion = hgetlo(tvc->m.DataVersion);
     a_result->callback = afs_data_pointer_to_int32(tvc->callback);		/* XXXX Now a pointer; change it XXXX */
     a_result->cbExpires = tvc->cbExpires;
@@ -178,6 +182,96 @@ fcnDone:
 
 } /*SRXAFSCB_GetCE*/
 
+int SRXAFSCB_GetCE64(a_call, a_index, a_result)
+    struct rx_call *a_call;
+    afs_int32 a_index;
+    struct AFSDBCacheEntry64 *a_result;
+
+{ /*SRXAFSCB_GetCE64*/
+
+    register int i;			/*Loop variable*/
+    register struct vcache *tvc;	/*Ptr to current cache entry*/
+    int code;				/*Return code*/
+    XSTATS_DECLS;
+
+#ifdef RX_ENABLE_LOCKS
+    AFS_GLOCK();
+#endif /* RX_ENABLE_LOCKS */
+
+    XSTATS_START_CMTIME(AFS_STATS_CM_RPCIDX_GETCE);
+
+    AFS_STATCNT(SRXAFSCB_GetCE64);
+    for (i = 0; i < VCSIZE; i++) {
+	for (tvc = afs_vhashT[i]; tvc; tvc = tvc->hnext) {
+	    if (a_index == 0)
+		goto searchDone;
+	    a_index--;
+	} /*Zip through current hash chain*/
+    } /*Zip through hash chains*/
+
+  searchDone:
+    if (tvc == (struct vcache *) 0) {
+	/*Past EOF*/
+	code = 1;
+	goto fcnDone;
+    }
+
+    /*
+     * Copy out the located entry.
+     */
+    a_result->addr = afs_data_pointer_to_int32(tvc);
+    a_result->cell = tvc->fid.Cell;
+    a_result->netFid.Volume = tvc->fid.Fid.Volume;
+    a_result->netFid.Vnode = tvc->fid.Fid.Vnode;
+    a_result->netFid.Unique = tvc->fid.Fid.Unique;
+    a_result->lock.waitStates = tvc->lock.wait_states;
+    a_result->lock.exclLocked = tvc->lock.excl_locked;
+    a_result->lock.readersReading = tvc->lock.readers_reading;
+    a_result->lock.numWaiting = tvc->lock.num_waiting;
+#if defined(INSTRUMENT_LOCKS)
+    a_result->lock.pid_last_reader = tvc->lock.pid_last_reader;
+    a_result->lock.pid_writer = tvc->lock.pid_writer;
+    a_result->lock.src_indicator = tvc->lock.src_indicator;
+#else
+    /* On osf20 , the vcache does not maintain these three fields */
+    a_result->lock.pid_last_reader = 0;
+    a_result->lock.pid_writer = 0;
+    a_result->lock.src_indicator = 0;
+#endif /* AFS_OSF20_ENV */
+#ifdef AFS_64BIT_ENV
+    a_result->Length = tvc->m.Length;
+#else /* AFS_64BIT_ENV */
+#ifdef AFS_64BIT_CLIENT
+    a_result->Length = tvc->m.Length;
+#else /* AFS_64BIT_CLIENT */
+    a_result->Length.high = 0;
+    a_result->Length.low = tvc->m.Length;
+#endif /* AFS_64BIT_CLIENT */
+#endif /* AFS_64BIT_ENV */
+    a_result->DataVersion = hgetlo(tvc->m.DataVersion);
+    a_result->callback = afs_data_pointer_to_int32(tvc->callback);		/* XXXX Now a pointer; change it XXXX */
+    a_result->cbExpires = tvc->cbExpires;
+    a_result->refCount = VREFCOUNT(tvc);
+    a_result->opens = tvc->opens;
+    a_result->writers = tvc->execsOrWriters;
+    a_result->mvstat = tvc->mvstat;
+    a_result->states = tvc->states;
+    code = 0;
+
+    /*
+     * Return our results.
+     */
+fcnDone:
+    XSTATS_END_TIME;
+
+#ifdef RX_ENABLE_LOCKS
+    AFS_GUNLOCK();
+#endif /* RX_ENABLE_LOCKS */
+
+    return(code);
+
+} /*SRXAFSCB_GetCE64*/
+
 
 /*------------------------------------------------------------------------
  * EXPORTED SRXAFSCB_GetLock
@@ -239,15 +333,9 @@ int SRXAFSCB_GetLock (a_call, a_index, a_result)
 	a_result->lock.exclLocked = ((struct afs_lock *)(tl->addr))->excl_locked;
 	a_result->lock.readersReading = ((struct afs_lock *)(tl->addr))->readers_reading;
 	a_result->lock.numWaiting = ((struct afs_lock *)(tl->addr))->num_waiting;
-#ifdef INSTRUMENT_LOCKS
 	a_result->lock.pid_last_reader = ((struct afs_lock *)(tl->addr))->pid_last_reader;
 	a_result->lock.pid_writer = ((struct afs_lock *)(tl->addr))->pid_writer;
 	a_result->lock.src_indicator = ((struct afs_lock *)(tl->addr))->src_indicator;
-#else
-	a_result->lock.pid_last_reader = 0;
-	a_result->lock.pid_writer = 0;
-	a_result->lock.src_indicator = 0;
-#endif
 	code = 0;
     }
 
@@ -1141,23 +1229,23 @@ int SRXAFSCB_GetCellServDB(
 
     /* search the list for the cell with this index */
     ObtainReadLock(&afs_xcell);
-
-    tcell = afs_GetCellByIndex(a_index, READ_LOCK, 0);
-
-    if (!tcell) {
-      i = 0;
-    } else {
-      p_name = tcell->cellName;
-      for (j = 0 ; j < AFSMAXCELLHOSTS && tcell->cellHosts[j] ; j++) {
-	a_hosts[j] = ntohl(tcell->cellHosts[j]->addr->sa_ip);
-      }
-      i = strlen(p_name);
-      afs_PutCell(tcell, READ_LOCK);
+    for (i=0, cq = CellLRU.next; cq != &CellLRU && i<= a_index; cq = tq, i++) {
+        tq = QNext(cq);
+	if (i == a_index) {
+	    tcell = QTOC(cq);
+	    p_name = tcell->cellName;
+	    for (j = 0 ; j < AFSMAXCELLHOSTS && tcell->cellHosts[j] ; j++) {
+		a_hosts[j] = ntohl(tcell->cellHosts[j]->addr->sa_ip);
+	    }
+	}
     }
 
+    if (p_name)
+	i = strlen(p_name);
+    else
+	i = 0;
     t_name = (char *)rxi_Alloc(i+1);
     if (t_name == NULL) {
-        ReleaseReadLock(&afs_xcell);
 #ifdef RX_ENABLE_LOCKS
 	AFS_GUNLOCK();
 #endif /* RX_ENABLE_LOCKS */
@@ -1217,7 +1305,6 @@ int SRXAFSCB_GetLocalCell(
      * the primary cell is when no other cell is explicitly marked as
      * the primary cell.  */
     ObtainReadLock(&afs_xcell);
-
     for (cq = CellLRU.next; cq != &CellLRU; cq = tq) {
         tq = QNext(cq);
 	tcell = QTOC(cq);
@@ -1236,7 +1323,6 @@ int SRXAFSCB_GetLocalCell(
 	plen = 0;
     t_name = (char *)rxi_Alloc(plen+1);
     if (t_name == NULL) {
-        ReleaseReadLock(&afs_xcell);
 #ifdef RX_ENABLE_LOCKS
 	AFS_GUNLOCK();
 #endif /* RX_ENABLE_LOCKS */
@@ -1371,3 +1457,74 @@ int SRXAFSCB_GetCacheConfig(
 
     return 0;
 }
+
+/*------------------------------------------------------------------------
+ * EXPORTED SRXAFSCB_FetchData
+ *
+ * Description:
+ *      Routine to do third party move from a remioserver to the original
+ *      issuer of an ArchiveData request. Presently supported only by the
+ *      "fs" command, not by the AFS client.
+ *
+ * Arguments:
+ *      rxcall:        Ptr to Rx call on which this request came in.
+ *      Fid:           pointer to AFSFid structure.
+ *      Fd:            File descriptor inside fs command.
+ *      Position:      Offset in the file.
+ *      Length:        Data length to transfer.
+ *      TotalLength:   Pointer to total file length field
+ *
+ * Returns:
+ *      0 on success
+ *
+ * Environment:
+ *      Nothing interesting.
+ *
+ * Side Effects:
+ *------------------------------------------------------------------------*/
+SRXAFSCB_FetchData(rxcall, Fid, Fd, Position, Length, TotalLength)
+    struct rx_call *rxcall;
+    struct AFSFid *Fid;
+    afs_int32 Fd;
+    afs_int64 Position;
+    afs_int64 Length;
+    afs_int64 *TotalLength;
+{
+    return ENOSYS;
+}
+
+/*------------------------------------------------------------------------
+ * EXPORTED SRXAFSCB_StoreData
+ *
+ * Description:
+ *      Routine to do third party move from a remioserver to the original
+ *      issuer of a RetrieveData request. Presently supported only by the
+ *      "fs" command, not by the AFS client.
+ *
+ * Arguments:
+ *      rxcall:        Ptr to Rx call on which this request came in.
+ *      Fid:           pointer to AFSFid structure.
+ *      Fd:            File descriptor inside fs command.
+ *      Position:      Offset in the file.
+ *      Length:        Data length to transfer.
+ *      TotalLength:   Pointer to total file length field
+ *
+ * Returns:
+ *      0 on success
+ *
+ * Environment:
+ *      Nothing interesting.
+ *
+ * Side Effects:
+ *      As advertised.
+ *------------------------------------------------------------------------*/
+SRXAFSCB_StoreData(rxcall, Fid, Fd, Position, Length, TotalLength)
+    struct rx_call *rxcall;
+    struct AFSFid *Fid;
+    afs_int32 Fd;
+    afs_int64 Position;
+    afs_int64 Length;
+    afs_int64 *TotalLength;
+{
+    return ENOSYS;
+}
