@@ -13,7 +13,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/afs_cell.c,v 1.7.2.6 2002/07/31 22:19:50 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/afs_cell.c,v 1.14 2001/11/13 20:31:31 shadow Exp $");
 
 #include "../afs/stds.h"
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
@@ -46,9 +46,9 @@ RCSID("$Header: /cvs/openafs/src/afs/afs_cell.c,v 1.7.2.6 2002/07/31 22:19:50 sh
 afs_rwlock_t afs_xcell;			/* allocation lock for cells */
 struct afs_q CellLRU;
 afs_int32 afs_cellindex=0;
-afs_int32 afs_realcellindex=0;
 afs_uint32 afs_nextCellNum = 0x100;
 
+
 /* Local variables. */
 struct cell *afs_rootcell = 0;
 
@@ -417,31 +417,6 @@ struct cell *afs_GetCellByIndex(cellindex, locktype, refresh)
 } /*afs_GetCellByIndex*/
 
 
-struct cell *afs_GetRealCellByIndex(cellindex, locktype, refresh)
-    register afs_int32 cellindex;
-    afs_int32 locktype;
-    afs_int32 refresh;
-{
-    register struct cell *tc;
-    register struct afs_q *cq, *tq;
-
-    AFS_STATCNT(afs_GetCellByIndex);
-    ObtainWriteLock(&afs_xcell,102);
-    for (cq = CellLRU.next; cq != &CellLRU; cq = tq) {
-	tc = QTOC(cq); tq = QNext(cq);
-	if (tc->realcellIndex == cellindex) {
-	    QRemove(&tc->lruq);
-	    QAdd(&CellLRU, &tc->lruq);
-	    ReleaseWriteLock(&afs_xcell);
-	    if (refresh) afs_RefreshCell(tc);
-	    return tc;
-	}
-    }
-    ReleaseWriteLock(&afs_xcell);
-    return (struct cell *) 0;
-} /*afs_GetRealCellByIndex*/
-
-
 afs_int32 afs_NewCell(acellName, acellHosts, aflags, linkedcname, fsport, vlport, timeout, aliasFor)
     int aflags;
     char *acellName;
@@ -509,11 +484,6 @@ afs_int32 afs_NewCell(acellName, acellHosts, aflags, linkedcname, fsport, vlport
 	tc->vlport = (vlport ? vlport : AFS_VLPORT);
 	afs_stats_cmperf.numCellsVisible++;
 	newc++;
-	if (!aflags & CAlias) {
-	    tc->realcellIndex = afs_realcellindex++;
-	} else {
-	    tc->realcellIndex = -1;
-	}
     }
 
     if (aflags & CLinkedCell) {
@@ -543,10 +513,7 @@ afs_int32 afs_NewCell(acellName, acellHosts, aflags, linkedcname, fsport, vlport
     tc->timeout = timeout;
 
     /* Allow converting an alias into a real cell */
-    if (!(aflags & CAlias)) {
-	tc->states &= ~CAlias;
-	tc->realcellIndex = afs_realcellindex++;
-    }
+    if (!(aflags & CAlias)) tc->states &= ~CAlias;
  
     memset((char *)tc->cellHosts, 0, sizeof(tc->cellHosts));
     if (aflags & CAlias) {
