@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/afs_daemons.c,v 1.7 2001/08/08 00:03:28 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/afs_daemons.c,v 1.13 2001/11/21 16:01:19 shadow Exp $");
 
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
 #include "../afs/afsincludes.h"	/* Afs-based standard headers */
@@ -28,6 +28,7 @@ short afs_brsWaiters = 0;	/* number of users waiting for brs buffers */
 short afs_brsDaemons = 0;	/* number of daemons waiting for brs requests */
 struct brequest	afs_brs[NBRS];	/* request structures */
 struct afs_osi_WaitHandle AFS_WaitHandler, AFS_CSWaitHandler;
+static int afs_brs_count = 0;	/* request counter, to service reqs in order */
 
 static int rxepoch_checked=0;
 #define afs_CheckRXEpoch() {if (rxepoch_checked == 0 && rxkad_EpochWasSet) { \
@@ -268,6 +269,7 @@ void afs_Daemon() {
 afs_CheckRootVolume () {
     char rootVolName[32];
     register struct volume *tvp;
+    int usingDynroot = afs_GetDynrootEnable();
 
     AFS_STATCNT(afs_CheckRootVolume);
     if (*afs_rootVolumeName == 0) {
@@ -276,7 +278,12 @@ afs_CheckRootVolume () {
     else {
 	strcpy(rootVolName, afs_rootVolumeName);
     }
-    tvp = afs_GetVolumeByName(rootVolName, LOCALCELL, 1, (struct vrequest *) 0, READ_LOCK);
+    if (usingDynroot) {
+	afs_GetDynrootFid(&afs_rootFid);
+	tvp = afs_GetVolume(&afs_rootFid, (struct vrequest *) 0, READ_LOCK);
+    } else {
+	tvp = afs_GetVolumeByName(rootVolName, LOCALCELL, 1, (struct vrequest *) 0, READ_LOCK);
+    }
     if (!tvp) {
 	char buf[128];
 	int len = strlen(rootVolName);
@@ -288,23 +295,25 @@ afs_CheckRootVolume () {
 	}
     }
     if (tvp) {
-	int volid = (tvp->roVol? tvp->roVol : tvp->volume);
-	afs_rootFid.Cell = LOCALCELL;
-	if (afs_rootFid.Fid.Volume && afs_rootFid.Fid.Volume != volid
-	    && afs_globalVp) {
-	    /* If we had a root fid before and it changed location we reset
-	     * the afs_globalVp so that it will be reevaluated.
-	     * Just decrement the reference count. This only occurs during
-	     * initial cell setup and can panic the machine if we set the
-	     * count to zero and fs checkv is executed when the current
-	     * directory is /afs.
-	     */
-	    AFS_FAST_RELE(afs_globalVp);
-	    afs_globalVp = 0;
+	if (!usingDynroot) {
+	    int volid = (tvp->roVol? tvp->roVol : tvp->volume);
+	    afs_rootFid.Cell = LOCALCELL;
+	    if (afs_rootFid.Fid.Volume && afs_rootFid.Fid.Volume != volid
+		&& afs_globalVp) {
+		/* If we had a root fid before and it changed location we reset
+		 * the afs_globalVp so that it will be reevaluated.
+		 * Just decrement the reference count. This only occurs during
+		 * initial cell setup and can panic the machine if we set the
+		 * count to zero and fs checkv is executed when the current
+		 * directory is /afs.
+		 */
+		AFS_FAST_RELE(afs_globalVp);
+		afs_globalVp = 0;
+	    }
+	    afs_rootFid.Fid.Volume = volid;
+	    afs_rootFid.Fid.Vnode = 1;
+	    afs_rootFid.Fid.Unique = 1;
 	}
-	afs_rootFid.Fid.Volume = volid;
-	afs_rootFid.Fid.Vnode = 1;
-	afs_rootFid.Fid.Unique = 1;
 	afs_initState = 300;    /* won */
 	afs_osi_Wakeup(&afs_initState);
 	afs_PutVolume(tvp, READ_LOCK);
@@ -338,7 +347,7 @@ afs_CheckRootVolume () {
     else return ENOENT;
 }
 
-/* parm 0 is the pathname, parm 1 to the fetch is the chunk number */
+/* ptr_parm 0 is the pathname, size_parm 0 to the fetch is the chunk number */
 void BPath(ab)
     register struct brequest *ab; {
     register struct dcache *tdc;
@@ -347,7 +356,7 @@ void BPath(ab)
 #ifdef AFS_LINUX22_ENV
     struct dentry *dp = NULL;
 #endif
-    afs_int32 offset, len;
+    afs_size_t offset, len;
     struct vrequest treq;
     afs_int32 code;
 
@@ -355,14 +364,14 @@ void BPath(ab)
     if (code = afs_InitReq(&treq, ab->cred)) return;
     AFS_GUNLOCK();
 #ifdef AFS_LINUX22_ENV
-    code = gop_lookupname((char *)ab->parm[0], AFS_UIOSYS, 1,  (struct vnode **) 0, &dp);
+    code = gop_lookupname((char *)ab->ptr_parm[0], AFS_UIOSYS, 1,  (struct vnode **) 0, &dp);
     if (dp)
 	tvn = (struct vnode*)dp->d_inode;
 #else
-    code = gop_lookupname((char *)ab->parm[0], AFS_UIOSYS, 1,  (struct vnode **) 0, (struct vnode **)&tvn);
+    code = gop_lookupname((char *)ab->ptr_parm[0], AFS_UIOSYS, 1,  (struct vnode **) 0, (struct vnode **)&tvn);
 #endif
     AFS_GLOCK();
-    osi_FreeLargeSpace((char *)ab->parm[0]); /* free path name buffer here */
+    osi_FreeLargeSpace((char *)ab->ptr_parm[0]); /* free path name buffer here */
     if (code) return;
     /* now path may not have been in afs, so check that before calling our cache manager */
     if (!tvn || !IsAfsVnode((struct vnode *) tvn)) {
@@ -386,7 +395,7 @@ void BPath(ab)
     tvc = (struct vcache *) tvn;
 #endif
     /* here we know its an afs vnode, so we can get the data for the chunk */
-    tdc = afs_GetDCache(tvc, ab->parm[1], &treq, &offset, &len, 1);
+    tdc = afs_GetDCache(tvc, ab->size_parm[0], &treq, &offset, &len, 1);
     if (tdc) {
 	afs_PutDCache(tdc);
     }
@@ -401,20 +410,21 @@ void BPath(ab)
 #endif
 }
 
-/* parm 0 to the fetch is the chunk number; parm 1 is the dcache entry to wakeup,
- * parm 2 is true iff we should release the dcache entry here.
+/* size_parm 0 to the fetch is the chunk number,
+ * ptr_parm 0 is the dcache entry to wakeup,
+ * size_parm 1 is true iff we should release the dcache entry here.
  */
 void BPrefetch(ab)
     register struct brequest *ab; {
     register struct dcache *tdc;
     register struct vcache *tvc;
-    afs_int32 offset, len;
+    afs_size_t offset, len;
     struct vrequest treq;
 
     AFS_STATCNT(BPrefetch);
     if (len = afs_InitReq(&treq, ab->cred)) return;
     tvc = ab->vnode;
-    tdc = afs_GetDCache(tvc, (afs_int32)ab->parm[0], &treq, &offset, &len, 1);
+    tdc = afs_GetDCache(tvc, ab->size_parm[0], &treq, &offset, &len, 1);
     if (tdc) {
 	afs_PutDCache(tdc);
     }
@@ -422,17 +432,18 @@ void BPrefetch(ab)
      * use tdc from GetDCache since afs_GetDCache may fail, but someone may
      * be waiting for our wakeup anyway.
      */
-    tdc = (struct dcache *) (ab->parm[1]);
-    tdc->flags &= ~DFFetchReq;
+    tdc = (struct dcache *) (ab->ptr_parm[0]);
+    ObtainSharedLock(&tdc->lock, 640);
+    if (tdc->mflags & DFFetchReq) {
+	UpgradeSToWLock(&tdc->lock, 641);
+	tdc->mflags &= ~DFFetchReq;
+	ReleaseWriteLock(&tdc->lock);
+    } else {
+	ReleaseSharedLock(&tdc->lock);
+    }
     afs_osi_Wakeup(&tdc->validPos);
-    if (ab->parm[2]) {
-#ifdef	AFS_SUN5_ENVX
-	mutex_enter(&tdc->lock);
-	tdc->refCount--;
-	mutex_exit(&tdc->lock);
-#else
+    if (ab->size_parm[1]) {
 	afs_PutDCache(tdc);	/* put this one back, too */
-#endif
     }
 }
 
@@ -504,13 +515,13 @@ int afs_BBusy() {
     return 1;
 }
 
-struct brequest *afs_BQueue(aopcode, avc, dontwait, ause, acred, aparm0, aparm1, aparm2, aparm3)
+struct brequest *afs_BQueue(aopcode, avc, dontwait, ause, acred, asparm0, asparm1, apparm0)
     register short aopcode;
     afs_int32 ause, dontwait;
     register struct vcache *avc;
     struct AFS_UCRED *acred;
-    /* On 64 bit platforms, "long" does the right thing. */
-    long aparm0, aparm1, aparm2, aparm3;
+    afs_size_t asparm0, asparm1;
+    void *apparm0;
 {
     register int i;
     register struct brequest *tb;
@@ -536,12 +547,12 @@ struct brequest *afs_BQueue(aopcode, avc, dontwait, ause, acred, aparm0, aparm1,
 #endif
 	    }
 	    tb->refCount = ause+1;
-	    tb->parm[0] = aparm0;
-	    tb->parm[1] = aparm1;
-	    tb->parm[2] = aparm2;
-	    tb->parm[3] = aparm3;
+	    tb->size_parm[0] = asparm0;
+	    tb->size_parm[1] = asparm1;
+	    tb->ptr_parm[0]  = apparm0;
 	    tb->flags = 0;
 	    tb->code = 0;
+	    tb->ts = afs_brs_count++;
 	    /* if daemons are waiting for work, wake them up */
 	    if (afs_brsDaemons > 0) {
 		afs_osi_Wakeup(&afs_brsDaemons);
@@ -729,7 +740,6 @@ afs_BioDaemon (nbiods)
     }
 
     /* Ignore HUP signals... */
-#ifdef AFS_AIX41_ENV
     {
  	sigset_t sigbits, osigbits;
  	/*
@@ -739,11 +749,6 @@ afs_BioDaemon (nbiods)
  	SIGDELSET(sigbits, SIGHUP);		/*   except SIGHUP	*/
  	limit_sigs(&sigbits, &osigbits);	/*   and already masked */
     }
-#else
-    SIGDELSET(u.u_procp->p_sig, SIGHUP);
-    SIGADDSET(u.u_procp->p_sigignore, SIGHUP);
-    SIGDELSET(u.u_procp->p_sigcatch, SIGHUP);
-#endif
     /* Main body starts here -- this is an intentional infinite loop, and
      * should NEVER exit 
      *
@@ -774,8 +779,16 @@ afs_BioDaemon (nbiods)
 	if (bp->b_flags & B_PFSTORE) {	/* XXXX */
 	    ObtainWriteLock(&vcp->lock,404);	    
 	    if (vcp->v.v_gnode->gn_mwrcnt) {
-		if (vcp->m.Length < bp->b_bcount + (u_int)dbtob(bp->b_blkno))
-		    vcp->m.Length = bp->b_bcount + (u_int)dbtob(bp->b_blkno);
+	    	afs_offs_t newlength = 
+			(afs_offs_t) dbtob(bp->b_blkno) + bp->b_bcount;
+		if (vcp->m.Length < newlength) {
+		    afs_Trace4(afs_iclSetp, CM_TRACE_SETLENGTH,
+			ICL_TYPE_STRING, __FILE__,
+			ICL_TYPE_LONG, __LINE__,
+			ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(vcp->m.Length),
+			ICL_TYPE_OFFSET, ICL_HANDLE_OFFSET(newlength));
+		    vcp->m.Length = newlength;
+		}
 	    }
 	    ReleaseWriteLock(&vcp->lock);
 	}
@@ -1084,21 +1097,9 @@ afs_BioDaemon (nbiods)
 
 
     /* Ignore HUP signals... */
-#ifdef AFS_AIX41_ENV
-    {
- 	sigset_t sigbits, osigbits;
- 	/*
- 	 * add SIGHUP to the set of already masked signals
- 	 */
- 	SIGFILLSET(sigbits);			/* allow all signals	*/
- 	SIGDELSET(sigbits, SIGHUP);		/*   except SIGHUP	*/
- 	limit_sigs(&sigbits, &osigbits);	/*   and already masked */
-    }
-#else
     SIGDELSET(u.u_procp->p_sig, SIGHUP);
     SIGADDSET(u.u_procp->p_sigignore, SIGHUP);
     SIGDELSET(u.u_procp->p_sigcatch, SIGHUP);
-#endif
     /* Main body starts here -- this is an intentional infinite loop, and
      * should NEVER exit 
      *
@@ -1216,6 +1217,9 @@ void afs_BackgroundDaemon() {
 
     MObtainWriteLock(&afs_xbrs,302);
     while (1) {
+	int min_ts;
+	struct brequest *min_tb;
+
 	if (afs_termState == AFSOP_STOP_BKG) {
 	    if (--afs_nbrs <= 0)
 		afs_termState = AFSOP_STOP_TRUNCDAEMON;
@@ -1223,42 +1227,50 @@ void afs_BackgroundDaemon() {
 	    afs_osi_Wakeup(&afs_termState);
 	    return;
 	}
-	
+
 	/* find a request */
 	tb = afs_brs;
 	foundAny = 0;
-	for(i=0;i<NBRS;i++,tb++) {
-	    /* look for request */
+	min_tb = NULL;
+	for(i=0; i<NBRS; i++, tb++) {
+	    /* look for request with smallest ts */
 	    if ((tb->refCount > 0) && !(tb->flags & BSTARTED)) {
 		/* new request, not yet picked up */
-		tb->flags |= BSTARTED;
-		MReleaseWriteLock(&afs_xbrs);
-		foundAny = 1;
-		afs_Trace1(afs_iclSetp, CM_TRACE_BKG1,
-			   ICL_TYPE_INT32, tb->opcode);
-		if (tb->opcode == BOP_FETCH)
-		    BPrefetch(tb);
-		else if (tb->opcode == BOP_STORE)
-		    BStore(tb);
-		else if (tb->opcode == BOP_PATH)
-		    BPath(tb);
-		else panic("background bop");
-		if (tb->vnode) {
-#ifdef	AFS_DEC_ENV
-		    tb->vnode->vrefCount--;	    /* fix up reference count */
-#else
-		    AFS_RELE((struct vnode *)(tb->vnode));	/* MUST call vnode layer or could lose vnodes */
-#endif
-		    tb->vnode = (struct vcache *) 0;
+		if ((min_tb && (min_ts - tb->ts > 0)) || !min_tb) {
+		    min_tb = tb;
+		    min_ts = tb->ts;
 		}
-		if (tb->cred) {
-		    crfree(tb->cred);
-		    tb->cred = (struct AFS_UCRED *) 0;
-		}
-		afs_BRelease(tb);   /* this grabs and releases afs_xbrs lock */
-		MObtainWriteLock(&afs_xbrs,305);
 	    }
 	}
+	if (tb = min_tb) {
+	    /* claim and process this request */
+	    tb->flags |= BSTARTED;
+	    MReleaseWriteLock(&afs_xbrs);
+	    foundAny = 1;
+	    afs_Trace1(afs_iclSetp, CM_TRACE_BKG1,
+		       ICL_TYPE_INT32, tb->opcode);
+	    if (tb->opcode == BOP_FETCH)
+		BPrefetch(tb);
+	    else if (tb->opcode == BOP_STORE)
+		BStore(tb);
+	    else if (tb->opcode == BOP_PATH)
+		BPath(tb);
+	    else panic("background bop");
+	    if (tb->vnode) {
+#ifdef	AFS_DEC_ENV
+		tb->vnode->vrefCount--;	    /* fix up reference count */
+#else
+		AFS_RELE((struct vnode *)(tb->vnode));	/* MUST call vnode layer or could lose vnodes */
+#endif
+		tb->vnode = (struct vcache *) 0;
+	    }
+	    if (tb->cred) {
+		crfree(tb->cred);
+		tb->cred = (struct AFS_UCRED *) 0;
+	    }
+	    afs_BRelease(tb);   /* this grabs and releases afs_xbrs lock */
+	    MObtainWriteLock(&afs_xbrs,305);
+	}
 	if (!foundAny) {
 	    /* wait for new request */
 	    afs_brsDaemons++;
@@ -1337,7 +1349,7 @@ afs_sgidaemon(void)
 			afs_sgibklist = NULL;
 			SPUNLOCK(afs_sgibklock, s);
 			AFS_GLOCK();
-			tdc->flags &= ~DFEntryMod;
+			tdc->dflags &= ~DFEntryMod;
 			afs_WriteDCache(tdc, 1);
 			AFS_GUNLOCK();
 			s = SPLOCK(afs_sgibklock);
