@@ -16,7 +16,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/afs_init.c,v 1.11.2.4 2002/11/15 13:28:32 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/afs_init.c,v 1.14 2002/02/16 18:23:49 shadow Exp $");
 
 #include "../afs/stds.h"
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
@@ -39,7 +39,6 @@ extern afs_rwlock_t afs_xaxs;
 extern afs_rwlock_t afs_xvolume;
 extern afs_rwlock_t afs_xuser;
 extern afs_rwlock_t afs_xserver;
-extern afs_rwlock_t afs_xsrvAddr;
 #ifndef AFS_AIX41_ENV
 extern afs_lock_t osi_fsplock;
 #endif
@@ -90,7 +89,6 @@ extern afs_int32 usedihint;
 
 /* afs_server.c */
 extern afs_int32 afs_setTime;
-extern afs_rwlock_t afs_xsrvAddr;
 
 /* Imported functions. */
 extern struct rx_securityClass *rxnull_NewServerSecurityObject();
@@ -225,6 +223,11 @@ afs_CacheInit(astatSize, afiles, ablocks, aDentries, aVolumes, achunk, aflags,
     afs_cacheStats = astatSize;
     afs_vcacheInit(astatSize);
     afs_dcacheInit(afiles, ablocks, aDentries, achunk, aflags);
+#ifdef AFS_64BIT_CLIENT
+#ifdef AFS_VM_RDWR_ENV
+    afs_vmMappingEnd = AFS_CHUNKBASE(0x7fffffff);
+#endif /* AFS_VM_RDWR_ENV */
+#endif /* AFS_64BIT_CLIENT */
 
 #if defined(AFS_AIX_ENV)
     {
@@ -523,8 +526,6 @@ afs_ResourceInit(preallocs)
     RWLOCK_INIT(&afs_xvolume, "afs_xvolume");
     RWLOCK_INIT(&afs_xcell, "afs_xcell");
     RWLOCK_INIT(&afs_xserver, "afs_xserver");
-    RWLOCK_INIT(&afs_xsrvAddr, "afs_xsrvAddr");
-    RWLOCK_INIT(&afs_icl_lock, "afs_icl_lock");
     RWLOCK_INIT(&afs_xinterface, "afs_xinterface");
     LOCK_INIT(&afs_puttofileLock, "afs_puttofileLock");
 #ifndef	AFS_AIX32_ENV
@@ -870,8 +871,10 @@ void shutdown_AFS()
       RWLOCK_INIT(&afs_xconn, "afs_xconn");
       memset((char *)&afs_rootFid, 0, sizeof(struct VenusFid));
       RWLOCK_INIT(&afs_xuser, "afs_xuser");
-      RWLOCK_INIT(&afs_xvolume, "afs_xvolume"), RWLOCK_INIT(&afs_xcell, "afs_xcell");
-      RWLOCK_INIT(&afs_xserver, "afs_xserver"), LOCK_INIT(&afs_puttofileLock, "afs_puttofileLock");
+      RWLOCK_INIT(&afs_xvolume, "afs_xvolume");
+      RWLOCK_INIT(&afs_xcell, "afs_xcell");
+      RWLOCK_INIT(&afs_xserver, "afs_xserver");
+      LOCK_INIT(&afs_puttofileLock, "afs_puttofileLock");
     }
     
 } /*shutdown_AFS*/
