@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/afs_osi.c,v 1.8.2.16 2004/10/18 06:04:18 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/afs_osi.c,v 1.14 2002/02/16 18:23:49 shadow Exp $");
 
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
 #include "../afs/afsincludes.h"	/* Afs-based standard headers */
@@ -91,10 +91,10 @@ register struct vcache *avc; {
     if (avc->opens > 0 || ((avc->v.v_flag & VTEXT) && !inode_uncache_try(avc))) return 1;
 #else
 #if defined(AFS_SGI_ENV)
-    if ((avc->opens > 0) || AFS_VN_MAPPED(AFSTOV(avc)))
+    if ((avc->opens > 0) || AFS_VN_MAPPED((struct vnode *)avc))
         return 1;
 #else
-    if (avc->opens > 0 || (AFSTOV(avc)->v_flag & VTEXT)) return(1);
+    if (avc->opens > 0 || (avc->v.v_flag & VTEXT)) return(1);
 #endif
 #endif /* AFS_MACH_ENV */
 #endif
@@ -289,28 +289,11 @@ void afs_osi_MaskSignals(){
     
 /* unmask signals in rxk listener */
 void afs_osi_UnmaskRxkSignals(){
+#ifdef AFS_LINUX22_ENV
+    osi_linux_unmask();
+#endif
 }
     
-/* Two hacks to try and fix afsdb */
-void 
-afs_osi_MaskUserLoop()
-{
-#ifdef AFS_DARWIN_ENV
-    afs_osi_Invisible();
-    afs_osi_fullSigMask();
-#else
-    afs_osi_MaskSignals();
-#endif
-}
-
-void 
-afs_osi_UnmaskUserLoop()
-{
-#ifdef AFS_DARWIN_ENV
-    afs_osi_fullSigRestore();
-#endif
-}
-
 /* register rxk listener proc info */
 void afs_osi_RxkRegister(){
 #ifdef AFS_LINUX22_ENV
@@ -419,18 +402,12 @@ afs_osi_SetTime(atv)
 #endif
 #ifdef AFS_HPUX_ENV
     {
-#if !defined(AFS_HPUX1122_ENV)
- /* drop the setting of the clock for now. spl7 is not
- * known on hpux11.22
- */
-
     register ulong_t s;
     struct timeval t;
     t.tv_sec = atv->tv_sec;
     t.tv_usec = atv->tv_usec;
     s = spl7(); time = t; (void) splx(s);
     resettodr(atv);
-#endif
     }
 #else
     {
@@ -467,7 +444,7 @@ void *afs_osi_Alloc(size_t x)
     AFS_STATS(afs_stats_cmperf.OutStandingAllocs++);
     AFS_STATS(afs_stats_cmperf.OutStandingMemUsage += x);
 #ifdef AFS_LINUX20_ENV
-    return osi_linux_alloc(x, 1);
+    return osi_linux_alloc(x);
 #else
     size = x;
     tm = (struct osimem *) AFS_KALLOC(size);
@@ -517,10 +494,6 @@ void afs_osi_Free(void *x, size_t asize)
 #endif
 }
 
-void afs_osi_FreeStr(char *x)
-{
-    afs_osi_Free(x, strlen(x) + 1);
-}
 
 /* ? is it moderately likely that there are dirty VM pages associated with 
  * this vnode?
@@ -826,33 +799,6 @@ void afs_osi_TraverseProcTable()
 }   
 #endif
 
-#if defined(AFS_LINUX22_ENV)
-void afs_osi_TraverseProcTable()
-{   
-    struct task_struct *p;
-
-#ifdef EXPORTED_TASKLIST_LOCK
-    read_lock(&tasklist_lock);
-#endif
-#ifdef DEFINED_FOR_EACH_PROCESS
-    for_each_process(p) if (p->pid) {
-        if (p->state & TASK_ZOMBIE)
-            continue;
-	afs_GCPAGs_perproc_func(p);
-    }
-#else
-    for_each_task(p) if (p->pid) {
-        if (p->state & TASK_ZOMBIE)
-            continue;
-	afs_GCPAGs_perproc_func(p);
-    }
-#endif
-#ifdef EXPORTED_TASKLIST_LOCK
-    read_unlock(&tasklist_lock);
-#endif
-}   
-#endif
-
 /* return a pointer (sometimes a static copy ) to the cred for a
  * given AFS_PROC.
  * subsequent calls may overwrite the previously returned value.
@@ -1034,29 +980,6 @@ const struct AFS_UCRED *afs_osi_proc2cred(AFS_PROC *pr)
     
     return rv;
 }  
-#elif defined(AFS_LINUX22_ENV)
-const struct AFS_UCRED *afs_osi_proc2cred(AFS_PROC *pr)
-{   
-    struct AFS_UCRED *rv=NULL;
-    static struct AFS_UCRED cr;
-
-    if(pr == NULL) {
-       return NULL;
-    }
-   
-    if ((pr->state == TASK_RUNNING) ||
-	(pr->state == TASK_INTERRUPTIBLE) ||
-	(pr->state == TASK_UNINTERRUPTIBLE) ||
-	(pr->state == TASK_STOPPED)) {
-	cr.cr_ref=1;
-	cr.cr_uid=pr->uid;
-	cr.cr_ngroups=pr->ngroups;
-	memcpy(cr.cr_groups, pr->groups, NGROUPS * sizeof(gid_t));
-	rv = &cr;
-    }
-    
-    return rv;
-}  
 #else
 const struct AFS_UCRED *afs_osi_proc2cred(AFS_PROC *pr)
 {
