@@ -72,9 +72,6 @@ struct xfs_inode_info {};
 #include <linux/string.h>
 #include <asm/semaphore.h>
 #include <linux/errno.h>
-#ifdef COMPLETION_H_EXISTS
-#include <linux/completion.h>
-#endif
 
 #else /* AFS_LINUX22_ENV */
 #if defined(AFS_DARWIN_ENV)
@@ -219,9 +216,6 @@ typedef unsigned short                  etap_event_t;
 struct vfspage;			/* for vnode.h compiler warnings */
 #    include "../h/swap.h"	/* for struct swpdbd, for vnode.h compiler warnings */
 #    include "../h/dbd.h"	/* for union idbd, for vnode.h compiler warnings */
-#ifdef AFS_HPUX110_ENV
-#    include "../h/resource.h"
-#endif
 #  endif /* AFS_HPUX_ENV */
 #if defined(AFS_DARWIN_ENV) || defined(AFS_FBSD40_ENV)
 #  include <sys/uio.h>
@@ -229,7 +223,7 @@ struct vfspage;			/* for vnode.h compiler warnings */
 #  include <sys/namei.h>
 #  include <sys/vnode.h>  
 #  include <sys/queue.h>    
-#  include <sys/malloc.h>    
+#  include <sys/malloc.h>
 #ifndef AFS_FBSD_ENV
 #  include <sys/ubc.h>
 #define timeout_fcn_t mach_timeout_fcn_t
@@ -273,13 +267,11 @@ MALLOC_DECLARE(M_AFS);
 #ifdef	AFS_SUN5_ENV
 #  include "../h/time.h"
 #else
-#if !defined(AFS_HPUX_ENV)
 #  include "../h/mbuf.h"
-#endif
 #endif /* AFS_SUN5_ENV */
 
 #include "../rpc/types.h"
-#include "../rx/xdr.h"
+#include "../rpc/xdr.h"
 
 #ifdef AFS_AIX32_ENV
 #  include "net/spl.h"
@@ -374,7 +366,7 @@ MALLOC_DECLARE(M_AFS);
 #undef register
 #endif	/* AFS_ALPHA_ENV */
 
-#include <rx/xdr.h>
+#include <rpc/xdr.h>
 #include <sys/proc.h>
 #include <sys/ioctl.h>
 
