@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/bozo/bosserver.c,v 1.12 2001/08/08 00:03:38 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/bozo/bosserver.c,v 1.14 2002/01/19 16:31:25 shadow Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -52,6 +52,9 @@ extern int BOZO_ExecuteRequest();
 extern int RXSTATS_ExecuteRequest();
 extern int afsconf_GetKey();
 extern struct bnode_ops fsbnode_ops, ezbnode_ops, cronbnode_ops;
+
+void bozo_Log();
+
 struct afsconf_dir *bozo_confdir = 0;	/* bozo configuration dir */
 static char *bozo_pid;
 struct rx_securityClass *bozo_rxsc[3];
@@ -666,9 +669,10 @@ char **envp;
     int noAuth = 0;
     struct ktc_encryptionKey tkey;
     int i;
-    pid_t	newSessionID;
     char namebuf[AFSDIR_PATH_MAX];
-
+#ifndef AFS_NT40_ENV
+    int nofork = 0;
+#endif
 #ifdef	AFS_AIX32_ENV
     struct sigaction nsa;
 
@@ -746,6 +750,9 @@ char **envp;
 	    DoSyslog = 1;
 	    DoSyslogFacility = atoi(argv[code]+8);
 	}
+	else if (strcmp(argv[code], "-nofork")==0) {
+	    nofork = 1;
+	}
 #endif
 	else if (strcmp(argv[code], "-enable_peer_stats")==0) {
 	    rx_enablePeerRPCStats();
@@ -766,6 +773,7 @@ char **envp;
 	    printf("Usage: bosserver [-noauth] [-log] "
 		   "[-syslog[=FACILITY]] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
+		   "[-nofork] "
 		   "[-help]\n");
 #else
 	    printf("Usage: bosserver [-noauth] [-log] "
@@ -809,6 +817,7 @@ char **envp;
     /* go into the background and remove our controlling tty */
 
 #ifndef AFS_NT40_ENV
+    if (!nofork)
     background();
 #endif /* ! AFS_NT40_ENV */
 
@@ -919,6 +928,7 @@ char **envp;
     rx_StartServer(1);	    /* donate this process */
 }
 
+void
 bozo_Log(a,b,c,d,e,f)
 char *a, *b, *c, *d, *e, *f; {
     char tdate[26];
