@@ -24,7 +24,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/fsprobe/fsprobe_callback.c,v 1.4.4.2 2003/03/06 14:53:40 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/fsprobe/fsprobe_callback.c,v 1.5 2001/11/01 04:02:25 shadow Exp $");
 
 #include <errno.h>
 #include <stdio.h>			/*Standard I/O stuff*/
@@ -210,7 +210,7 @@ afs_int32 SRXAFSCB_Probe(rxcall)
 
 
 /*------------------------------------------------------------------------
- * SRXAFSCB_GetCE
+ * SRXAFSCB_GetCE64
  *
  * Description:
  *	Respond minimally to a request for returning the contents of
@@ -230,6 +230,29 @@ afs_int32 SRXAFSCB_Probe(rxcall)
  *	As advertised.
  *------------------------------------------------------------------------*/
 
+afs_int32 SRXAFSCB_GetCE64(rxcall)
+    struct rx_call *rxcall;
+
+{ /*SRXAFSCB_GetCE64*/
+
+#if XSTAT_FS_CALLBACK_VERBOSE
+    static char rn[] = "SRXAFSCB_GetCE64";		/*Routine name*/
+    char hostName[256];				/*Host name buffer*/
+    char *hostNameResult;			/*Ptr to static*/
+
+    if (rxcall != (struct rx_call *)0) {
+	hostNameResult =
+	    hostutil_GetNameByINet((afs_int32)(rxcall->conn->peer->host));
+	strcpy(hostName, hostNameResult);
+	fprintf(stderr, "[%s:%s] Called from host %s, port %d\n",
+		mn, rn, hostName, rxcall->conn->peer->port);
+    } /*Valid rxcall param*/
+#endif /* XSTAT_FS_CALLBACK_VERBOSE */
+
+    return(0);
+
+} /*SRXAFSCB_GetCE64*/
+
 afs_int32 SRXAFSCB_GetCE(rxcall)
     struct rx_call *rxcall;
 
@@ -641,16 +664,7 @@ int SRXAFSCB_GetCellServDB(
     struct rx_call *a_call,
     afs_int32 a_index,
     char **a_name,
-    serverList *a_hosts)
-{
-    return RXGEN_OPCODE;
-}
-
-afs_int32 SRXAFSCB_GetCellByNum(
-    struct rx_call *a_call,
-    afs_int32 a_cellnum,
-    char **a_name,
-    serverList *a_hosts)
+    afs_int32 *a_hosts)
 {
     return RXGEN_OPCODE;
 }
