@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/venus/fs.c,v 1.12.2.2 2002/08/28 03:53:33 zacheiss Exp $");
+RCSID("$Header: /cvs/openafs/src/venus/fs.c,v 1.13 2001/11/01 05:24:39 shadow Exp $");
 
 #include <afs/afs_args.h>
 #include <rx/xdr.h>
@@ -1909,8 +1909,6 @@ static NewCellCmd(as)
      * MAXCELLHOSTS (8) servers. To determine which we are talking to,
      * do a GETCELL pioctl and pass it a magic number. If an array of
      * 8 comes back, its a 3.5 client. If not, its a 3.4 client.
-     * If we get back EDOM, there are no cells in the kernel yet,
-     * and we'll assume a 3.5 client.
      */
     tp = space;
     lp = (afs_int32 *)tp;
@@ -1921,17 +1919,13 @@ static NewCellCmd(as)
     blob.in       = space;
     blob.out      = space;
     code = pioctl(0, VIOCGETCELL, &blob, 1);
-    if (code < 0 && errno != EDOM) {
+    if (code < 0) {
        Die(errno, 0);
        return 1;
     }
-    if (code < 1 && errno == EDOM) {
-       scount = MAXHOSTS;
-    } else {
-       tp = space;
-       cellname = tp + MAXCELLHOSTS*sizeof(afs_int32);
-       scount = ((cellname[0] != '\0') ? MAXCELLHOSTS : MAXHOSTS);
-    }
+    tp = space;
+    cellname = tp + MAXCELLHOSTS*sizeof(afs_int32);
+    scount = ((cellname[0] != '\0') ? MAXCELLHOSTS : MAXHOSTS);
 
     /* Now setup and do the NEWCELL pioctl call */
     memset(space, 0, (scount+1) * sizeof(afs_int32));
