@@ -19,7 +19,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/vol/volume.c,v 1.12.2.4 2002/11/14 20:41:44 zacheiss Exp $");
+RCSID("$Header: /cvs/openafs/src/vol/volume.c,v 1.13 2001/09/13 23:19:24 rees Exp $");
 
 #include <rx/xdr.h>
 #include <afs/afsint.h>
@@ -130,6 +130,9 @@ RCSID("$Header: /cvs/openafs/src/vol/volume.c,v 1.12.2.4 2002/11/14 20:41:44 zac
 #endif /* AFS_PTHREAD_ENV */
 #include "vutils.h"
 #include "fssync.h"
+#if !defined(AFS_NT40_ENV) && !defined(AFS_NAMEI_ENV)
+#include <afs/osi_inode.h>
+#endif
 #ifndef AFS_NT40_ENV
 #include <unistd.h>
 #endif
@@ -388,11 +391,6 @@ static void ReadHeader(Error *ec, IHandle_t *h, char *to, int size,
     FdHandle_t *fdP;
 
     *ec = 0;
-    if (h == NULL) {
-	*ec = VSALVAGE;
-	return;
-    }
-
     fdP = IH_OPEN(h);
     if (fdP == NULL) {
 	*ec = VSALVAGE;
